/**********************************************************************
 *
 * cfgread.cpp -- 
 * Copyright (C) 1999  The New Zealand Digital Library Project
 *
 * A component of the Greenstone digital library software
 * from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *********************************************************************/

#include "cfgread.h"
#include <ctype.h> // for isspace()

int write_ini_line (ofstream &fileout, const text_t &key, const text_t value) {
  if (key.empty() || value.empty()) return -1;
  outconvertclass text_t2ascii;
  fileout << text_t2ascii << key << "=" << value << "\n";
  return 0;
}

int read_ini_line (ifstream &filein, text_t &key, text_t &value) {
  if (filein.eof()) return -1;

  key.clear();
  value.clear();
  char c;
  filein.get(c);

  int foundeq = 0;
  while (!filein.eof() && c != '\n') {
    if (!foundeq && c == '=') {foundeq = 1; filein.get(c);}

    if (foundeq) value.push_back(c);
    else key.push_back(c);
    filein.get(c);
  }
  if (key.empty()) return 0; // blank line maybe?
  return 0;
} 

// write out line of values to cfgfile
// does nothing fancy - make sure no values contain carriage returns
int write_cfg_line (ofstream &fileout, const text_tarray &values) {
  outconvertclass text_t2ascii;

  text_tarray::const_iterator here = values.begin();
  text_tarray::const_iterator end = values.end();

  bool first = true;
  while (here != end) {
    if (first) fileout << text_t2ascii << *here;
    else fileout << text_t2ascii << "  \"" << *here << "\"";
    first = false;
    here ++;
  }
  fileout << "\n";
  return 0;
}

// same, but use a file descriptor this time
int write_cfg_line (int fileout, const text_tarray &values) {
  outconvertclass text_t2ascii;

  text_tarray::const_iterator here = values.begin();
  text_tarray::const_iterator end = values.end();

  if (here != end) {
    char *s=here->getcstr();
    write(fileout,s ,here->size());
    delete s;

    ++here;
  }
  while (here != end) {
    write(fileout, " \"", 2);
    char *s=here->getcstr();
    write(fileout,s ,here->size());
    delete s;
    write(fileout, "\"", 1);

    ++here;
  }
  write(fileout,"\n",1);
  return 0;
}


// returns 0 on success, -1 on failure
int read_cfg_line (ifstream &filein, text_tarray &values) {
  // we split up the line into tokens, pushing each token into the
  // values array. Quoted phrases are a single token. A "\" at the end
  // of a line continues onto the next line.

  values.erase(values.begin(), values.end());

  if (!filein.good()) return -1;

  text_t curvalue;
  char c;
  filein.get(c);

  // skip white space
  while (!filein.eof() && isspace(c)) { filein.get(c); }

    // ignore comments
  while (c == '#') {
    while (!filein.eof() && c!='\n' && c!='\r') { filein.get(c); }
    // skip white space...
    while (!filein.eof() && isspace(c)) { filein.get(c); }
  }

  // deal with all the records on this line (possibly multi-line)
  while (!filein.eof()) {
    if (c=='\n' || c=='\r') { // shouldn't happen?
      break;
    }

    // get the next token
    curvalue.clear();
      
      // see if this is a quoted phrase
    if (c=='\'' || c=='\"') { // starts with a quote
      char quote, old_c;
      quote = c;
      old_c = c;
      filein.get(c);
      while (!filein.eof() && (c != quote || old_c == '\\') ) {
	/* Turn eol into space, in case other parsing bits expect eol to
	   also mean end of parsing... */
	if (c=='\r' || c=='\n') c=' ';
	curvalue.push_back(c);
	old_c = c;
	filein.get(c);
      }
      // get the character after the closing quote...
      filein.get(c);
    } else { // it's not a quoted phrase
      // get the token
      while (!filein.eof() && !isspace(c)) {
	curvalue.push_back(c);
	filein.get(c);
      }
    }
    // we now have a token or a phrase
    
    // see if we've reached the end of the line
    if (c == '\n' || c == '\r') {
      if (curvalue != "\\") { // the line DOESN'T continue. End of line.
	values.push_back(curvalue);
	break;
      } else {
	// swallow up the EOL chars
	while (!filein.eof() && (c=='\r' || c=='\n')) filein.get(c);
      }
    } else { // no new line seen
      values.push_back(curvalue);
    }

    curvalue.clear();

    // remove whitespace (but not newline/CR chars) before next token
    while (!filein.eof() && (c==' ' || c=='\t')) filein.get(c);

  } // while(1)
  

  return 0;
}
